import { createContext, useContext, useEffect, useState, useRef, useCallback, useMemo } from 'react';
import { checkRequestState } from '@proscom/prostore';

var ProstoreContext = createContext(null);
function useContextStore(storeOrName) {
    var context = useContext(ProstoreContext);
    if (typeof storeOrName === 'string') {
        var store = context[storeOrName];
        if (!store) {
            throw new Error("Store '" + storeOrName + "' is not registered in the context");
        }
        return store;
    }
    return storeOrName;
}

var ProstoreSsrContext = createContext(null);
var ProstoreSsrManager = /** @class */ (function () {
    function ProstoreSsrManager(_a) {
        var isServer = _a.isServer, initialStates = _a.initialStates;
        this.isServer = false;
        this.states = {};
        this.promises = new Set();
        this.disabled = false;
        this.isServer = isServer || false;
        this.states = initialStates || this.states;
    }
    ProstoreSsrManager.prototype.registerPromise = function (promise) {
        var _this = this;
        this.promises.add(promise);
        promise.then(function () {
            _this.promises.delete(promise);
        });
    };
    ProstoreSsrManager.prototype.hasPromises = function () {
        return this.promises.size > 0;
    };
    ProstoreSsrManager.prototype.waitForCurrentPromises = function () {
        return Promise.all(this.promises);
    };
    ProstoreSsrManager.prototype.getState = function (key) {
        if (this.disabled)
            return null;
        return this.states[key];
    };
    ProstoreSsrManager.prototype.setState = function (key, value) {
        if (this.disabled)
            return;
        this.states[key] = value;
    };
    ProstoreSsrManager.prototype.getStates = function () {
        return this.states;
    };
    ProstoreSsrManager.prototype.setDisabled = function (value) {
        this.disabled = value;
    };
    return ProstoreSsrManager;
}());

/**
 * Подписка на стор, чтобы актуализировать локальную копию
 */
function useConnectStore(store, setState) {
    useEffect(function () {
        var sub = store.state$.subscribe(function (state) {
            setState(state);
        });
        return function () {
            sub.unsubscribe();
        };
    }, [store, setState]);
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

/**
 * Хук, подключающий компонент к стору и запрашивающий данные
 * в соответствии с variables
 */
function useRequestStore(storeOrName, variables, options) {
    if (options === void 0) { options = null; }
    // Подключаем стор из контекста
    var store = useContextStore(storeOrName);
    // Подключение к контексту, который хранит все результаты
    var ssrContext = useContext(ProstoreSsrContext);
    // Фейковое состояние для ререндера
    var _a = useState(0), rerender = _a[1];
    // Локальная копия состояния стора
    var stateRef = useRef(null);
    // Ссылка на текущий стор
    var storeRef = useRef(null);
    // Если стор изменился
    if (store !== storeRef.current) {
        // Запоминаем текущий стор
        storeRef.current = store;
        // Если в контексте есть значение, используем его
        stateRef.current = store.getValueFromContext(ssrContext);
    }
    // Колбек для обновления состояния
    var setState = useCallback(function (newState) {
        if (stateRef.current !== newState) {
            stateRef.current = newState;
            rerender(function (s) { return s + 1; });
        }
    }, []);
    // Подписываемся на изменения стора
    useConnectStore(store, setState);
    // Локальная копия состояния стора
    var state = stateRef.current;
    // Проверка, нужно ли запрашивать данные
    var queryCheck = checkRequestState(state, variables);
    // Мемоизированный колбек для загрузки данных
    var loadData = useCallback(function () {
        return store.loadData(variables, __assign({ ssr: ssrContext && ssrContext.isServer }, options)).then(function () {
            store.setContextValue(ssrContext);
        });
    }, [store, variables, options]);
    // Запрос данных, если он нужен
    useEffect(function () {
        if (queryCheck.request) {
            // Вероятность ошибки в этом месте мала, т.к.
            // основные ошибки обрабатываются в RequestStore
            // Поэтому мы ее просто залогируем
            loadData().catch(function (e) { return console.error(e); });
        }
    }, [queryCheck, loadData]);
    // Если мы сейчас в режиме серверного рендеринга, то запрашиваем данные
    if (ssrContext && ssrContext.isServer && queryCheck.request && store.ssrId) {
        ssrContext.registerPromise(loadData());
    }
    // Возвращаем то, что может быть полезно в самом компоненте
    return { check: queryCheck, state: state, load: loadData };
}
function useRequestStoreVars(storeOrName, variableCreator, args) {
    var variables = useMemo(variableCreator, args);
    return useRequestStore(storeOrName, variables);
}

/**
 * Хук, подключающий компонент к стору
 */
function useStore(storeOrName) {
    var store = useContextStore(storeOrName);
    var _a = useState(store.state), state = _a[0], setState = _a[1];
    useConnectStore(store, setState);
    return state;
}
/**
 * Хук, подключащий компонент к стору и возвращающий часть состояния
 */
function useStoreSelect(storeOrName, select) {
    var state = useStore(storeOrName);
    return useMemo(function () { return select(state); }, [select, state]);
}

export { ProstoreContext, ProstoreSsrContext, ProstoreSsrManager, useConnectStore, useContextStore, useRequestStore, useRequestStoreVars, useStore, useStoreSelect };
